<?php

class Op5reports_options extends Report_options
{
	public static $right_sub_type = false;
	protected $subs = array();

	public function __construct($options=false)
	{
		$this->vtypes = array(
			'use_pnp' => array('type' => 'bool', 'default' => false),
			'use_summary' => array('type' => 'bool', 'default' => false),
		);
		parent::__construct($options);
		if (is_a($options, 'Op5reports_options')) {
			foreach ($options->subs as $sub) {
				$class = get_class($sub);
				$type = substr($class, 0, strpos($class, '_'));
				$this->add_sub($type, $sub);
			}
		} else {
			$this->add_sub('summary', $options);
		}
	}

	public function add_sub($type, $options)
	{
		$sub = static::setup_options_obj($type, $options);
		$this->options = array_merge($this->options, $sub->options);
		array_unshift($this->subs, $sub);
	}

	public function set($name, $value)
	{
		parent::set($name, $value);
		foreach ($this->subs as $sub) {
			if ($sub->set($name, $value)) {
				# handle weird side-effects
				$this->options = array_merge($this->options, $sub->options);
			}
		}
	}

	public function get_alternatives($key)
	{
		$out = parent::get_alternatives($key);
		if ($out !== false)
			return $out;
		$current = false;
		foreach ($this->subs as $sub) {
			$out = $sub->get_alternatives($key);
			if ($out !== false) {
				if($out) {
					// return the first non-falsy value
					return $out;
				}
				$current = $out; // might for example be an empty array or a '0'
			}
		}
		return $current;
	}

	public function offsetGet($str)
	{
		$out = parent::offsetGet($str);
		if ($out !== false)
			return $out;
		$current = false;
		foreach ($this->subs as $sub) {
			$out = $sub->offsetGet($str);
			if ($out !== false) {
				if($out) {
					// return the first non-falsy value
					return $out;
				}
				$current = $out; // might for example be an empty array or a '0'
			}
		}
		return $current;
	}

	public function get_value($key)
	{
		$out = parent::get_value($key);
		if ($out !== false)
			return $out;
		$current = false;
		foreach ($this->subs as $sub) {
			$out = $sub->get_value($key);
			if ($out !== false) {
				if($out) {
					// return the first non-falsy value
					return $out;
				}
				$current = $out; // might for example be an empty array or a '0'
			}
		}
		return $current;
	}

	function next() {
		while (true) {
			$x = next($this->options);
			if ($x === false)
				break;
			if (isset($this->vtypes[key($this->options)]) && !isset($this->vtypes[key($this->options)]['generated']))
				break;
			foreach ($this->subs as $sub) {
				if (isset($sub->vtypes[key($this->options)]) && !isset($sub->vtypes[key($this->options)]['generated']))
					break 2;
			}
		}
		return $x;
	}

	protected static function discover_options($type, $input = false)
	{
		return parent::discover_options(self::$right_sub_type, $input);
	}
}
